﻿/* ============================================================
* Author:       xieqin
* Time:         2014/11/11 11:18:27
* FileName:     EventSystem
* Purpose:      eventSystem
* =============================================================*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Reflection;

namespace Event
{
    //事件系统实现
    public class EventSystem : IEventSystem
    {
        private Dictionary<int, Dictionary<int,Action<IEventBase>>> mapHandlers = new Dictionary<int, Dictionary<int, Action<IEventBase>>>();
        private int uuid = 1;
        int GenerateUuid()
        {
            return uuid++;
        }

        public EventTag AddListener<T>(int eventId, Action<T> handler) where T : class, IEventBase
        {
            Action<IEventBase> callback = delegate(IEventBase e)
            {
                handler(e as T);
            };

            EventTag tag = new EventTag(eventId, GenerateUuid());

            if (!mapHandlers.ContainsKey(eventId))
            {
                mapHandlers[eventId] = new Dictionary<int,Action<IEventBase>>();
            }

            var mapCallbacks = mapHandlers[eventId];
            mapCallbacks.Add(tag.SessionId, callback);

            return tag;
        }

        public EventTag AddListener<T>(Action<T> handler) where T : class, IEventBase, new()
        {
            T ev = new T();
            return AddListener(ev.Id, handler);
        }

        public void RemoveListener(EventTag tag)
        {
            if (!mapHandlers.ContainsKey(tag.EventId))
            {
                return;
            }

            var mapCallbacks = mapHandlers[tag.EventId];
            mapCallbacks.Remove(tag.SessionId);
        }

        public void SendEvent(IEventBase ev)
        {
            DispatchEvent(ev);
        }

        public void Destroy()
        {
            mapHandlers.Clear();
        }

        private void DispatchEvent(IEventBase ev)
        {
            if (mapHandlers.ContainsKey(ev.Id))
            {
                var mapCallbacks = mapHandlers[ev.Id];
                foreach(var callback in mapCallbacks.Values)
                {
                    callback(ev);
                }
            }
        }
    }
}
